#include <gamekit.h>
#include <avr/pgmspace.h>

uint8_t img[5][7] PROGMEM ={
  0 ,3 ,3 ,3 ,3 ,3 ,0 ,
  3 ,0 ,0 ,0 ,0 ,0 ,3 ,
  3 ,0 ,0 ,0 ,0 ,0 ,3 ,
  3 ,0 ,0 ,0 ,0 ,0 ,3 ,
  0 ,3 ,3 ,3 ,3 ,3 ,0 
};



/*
The get_buttons() function is to easily read all button states at once, and without any preprocessing like button_pressed() does.
 get_buttons() returns a 8-bit unsigned int (uint8_t) in which the bits represents each a state of a button (the remaining two bits read always 0).
 
 This function can be used in a switch statement, for example:
 
 switch(get_buttons())
 {
 	case butt_UP:
 	{
 		//some code to be executed when button up is pressed
 	};break;
 
 	case butt_DOWN:
 	{
 		//some code to be executed when button down is pressed
 	};break;
 
 	case (butt_UP | butt_DOWN):
 	{
 		//some code to be executed when button up and down is pressed at once
 	};break;
 
 	case butt_FUNCA:
 	case butt_FUNCB:
 	{
 		//some code to be executed when button FUNC A or FUNC B is pressed
 	};break;
 
 
 }
 
 BUT, there a few things to know:
 - please use the butt_**** macros to get the button numbers. This is for code readability and hardware compatibility (the order of bits in the return of get_buttons() is differs in some hardware versions)
 - there is no postprocessing like button_pressed() does it (no debouncing, no repeat, nothing‚Ä¶)
 - The get_buttons() function returns all button states at once, so you have to have cases for all combinations of buttons that may be important to you. That's why in the switch example you see a case (butt_UP | butt_DOWN), it will be executed when both buttons will be pressed at once.
 - you may have just one code for more cases, as you see in the example for FUNC A and FUNC B buttons. This is like a "OR" connection.
 */



void setup(){
  gamekit.Begin();
  gamekit.load_image( img );

  Serial.begin(9600);
}

byte old_buttons = 0;

void loop(){

  switch(gamekit.get_buttons())
  {
  case butt_UP:
    {
      gamekit.set_pixel(1, 1, 15);
    };
    break;

  case butt_DOWN:
    {
      gamekit.set_pixel(2, 2, 15);
    };
    break;

    case (butt_UP | butt_DOWN):
    {
      gamekit.set_pixel(3, 3, 15);
    };
    break;

  case butt_FUNCA:
  case butt_FUNCB:
    {
      gamekit.load_image( img );
    };
    break;
  }

  if(old_buttons != gamekit.get_buttons())
  {
    old_buttons = gamekit.get_buttons();

    Serial.println("some button has changed its state!");
  }

}




