/*************************************************************************

	gamekit library
	
This library is to provide a user API for the mignon gamekit V 2.0

	http://www.mignongamekit.com

**************************************************************************/

#include <gamekit.h>
#include <avr/pgmspace.h>
//#include <wiring.h>
#include "Arduino.h"

gamekit_class gamekit;

#ifdef __AVR_ATmega328P__

#define PB0 PORTB0
#define PB1 PORTB1
#define PB2 PORTB2
#define PB3 PORTB3
#define PB4 PORTB4
#define PB5 PORTB5
#define PB6 PORTB6
#define PB7 PORTB7

#define PD0 PORTB0
#define PD1 PORTB1
#define PD2 PORTB2
#define PD3 PORTB3
#define PD4 PORTB4
#define PD5 PORTB5
#define PD6 PORTB6
#define PD7 PORTB7

#define PC0 PORTC0
#define PC1 PORTC1
#define PC2 PORTC2
#define PC3 PORTC3
#define PC4 PORTC4
#define PC5 PORTC5

#endif

/******************************************************************
	soundboard related funtions
******************************************************************/

uint8_t gamekit_class::get_soundboard_volume()
{
    uint16_t tmp = getadc(5);

    tmp = 511 - tmp;
    tmp = tmp / 2;

    return (uint8_t) tmp;
}

uint16_t gamekit_class::get_soundboard_frequency()
{
	uint8_t save = melody_is_playing;
	
	melody_is_playing = 0;

	if(!gamekit_soundboard_freq_active)
	{
		TCCR1A = 0;
		TCCR1B = (1 << CS11) | (1 << CS12);
		gamekit_soundboard_freq_active = 1;
	}
	
	while(!gamekit_soundboard_freq_valid);

	melody_is_playing = save;

	return (uint16_t) ( gamekit_soundboard_freq_tmp * 8.4388186 );
}

uint16_t gamekit_class::getadc(uint8_t channel)
{
	ADMUX = (channel & 0b00000111) | (1 << REFS0); // nur kanäle 0-7 zulassen
	ADCSRA |= (1 << ADSC); // ADC starten		
	
	do
	{
		asm volatile ("nop");
	} while ((ADCSRA & (1 << ADSC)) != 0);
	
	return ADC;
}

/******************************************************************
	sound output related funtions
******************************************************************/

//return the OCR1A value to a given frequency (used to play soundes)
uint16_t gamekit_class::freq2ocr(uint16_t freq)
{
    return (uint16_t) ((F_CPU / (2 * 8 * freq)) - 1);
}

//play the tone "freq" Hz for "time" ms
void gamekit_class::play_tone(uint16_t freq, uint16_t time, uint8_t loudness)
{
    uint8_t save = melody_is_playing;
    
    melody_is_playing = 0;

    if (loudness == LOUD) {
	DDR(PORTD) |= _BV(PD4);
	PORTD |= _BV(PD4);
    } else {
	DDR(PORTD) &= ~_BV(PD4);
	PORTD &= ~_BV(PD4);
    };

    unsigned long old = millis();

    start_tone(freq);
    
    while ((old + time) >= millis())
	asm volatile ("nop");

    stop_tone();
    
    melody_is_playing = save;
}

void gamekit_class::play_melody(uint16_t * melody)
{
    gamekit_soundboard_freq_active = 0;

    if (melody_is_playing == 1) {
	melody_is_playing = 0;
	uint32_t old_system_counter = system_counter;
	while (system_counter <= (uint32_t) (old_system_counter + 1))
	    asm volatile ("nop");
    };

    current_melody_event = 0;
    melody_is_playing = 1;
    melody_ptr = melody;
}

void gamekit_class::start_tone(uint16_t freq)
{
    gamekit_soundboard_freq_active = 0;

    TCCR1A = (_BV(COM1A0));	// WGM = CTC
    TCCR1B = (_BV(WGM12) | _BV(CS11));	// Prescaler = 8; Timer1 aktivieren

    OCR1A = freq2ocr(freq);

    DDR(PORTB) |= _BV(PB1);	//Speaker auf Ausgang
}

void gamekit_class::stop_tone()
{
    gamekit_soundboard_freq_active = 1;
    DDR(PORTB) &= ~_BV(PB1);
    TCCR1A = 0;
    TCCR1B = (1 << CS11) | (1 << CS12);
}

uint16_t gamekit_class::get_current_melody_event()
{
     return current_melody_event;
}

void gamekit_class::set_current_melody_event(uint16_t event)
{
     current_melody_event = event;
}



/******************************************************************
	button related funtions
******************************************************************/

//generic function to get states of all buttons; this can be useful for switch statements
uint8_t gamekit_class::get_buttons()
{
    return (~buttons_raw & 0x3F);
}

//function that returns one time true if button "button" is pressed and a "series" of trues if button is holded (see examples for more information)
boolean gamekit_class::button_pressed(uint8_t button)
{
    if ((buttons_pp & button) == 0) {
	buttons_pp |= button;
	return true;
    } else
	return false;
}

//this function just waits until the button "button" is pressed; warning: this can cause a deadlock if button is never pressed!
void gamekit_class::wait_button_pressed(uint8_t button)
{
    while ((buttons_raw & button) != 0)
	asm volatile ("nop");
}

//this function just waits until the button "button" is released; warning: this can cause a deadlock if button is never released!
void gamekit_class::wait_button_released(uint8_t button)
{
    while ((buttons_raw & button) != button)
	asm volatile ("nop");
}

//this function just waits until the button "button" is released; warning: this can cause a deadlock if button is never released!
void gamekit_class::set_button_timing(uint8_t first, uint8_t common)
{
   gamekit_button_first_waittime = first;
   gamekit_button_waittime = common;
}

/******************************************************************
	functions related to the value of a pixel
******************************************************************/

//sets the pixel at (r,c) to the value v
void gamekit_class::set_pixel(uint8_t r, uint8_t c, uint8_t v)
{
    if ((c < gamekit_row_count) && (r < gamekit_col_count))
	frame[r][c] = v;
}

//returns the vaule of the pixel at (r,c)
uint8_t gamekit_class::get_pixel(uint8_t r, uint8_t c)
{
    if ((c < gamekit_row_count) && (r < gamekit_col_count))
	return frame[r][c];
    else
	return 0;
}

//loads the array img from flash to the display
void gamekit_class::load_image(const uint8_t img[gamekit_col_count][gamekit_row_count])
{
    for (uint8_t r = 0; r < gamekit_row_count; r++)
	for (uint8_t c = 0; c < gamekit_col_count; c++) {
	    frame[c][r] = pgm_read_byte(&img[c][r]);
	};
}


//loads the array img from flash to the display
void gamekit_class::load_map(uint8_t * img, uint8_t r_count, uint8_t c_count, uint8_t r_off, uint8_t c_off)
{
    if ((r_off + gamekit_col_count <= r_count) && (c_off + gamekit_row_count <= c_count))
	for (uint8_t c = 0; c < gamekit_col_count; c++)
	    for (uint8_t r = 0; r < gamekit_row_count; r++) {
		frame[c][r] = pgm_read_byte(&img[((c + r_off) * c_count) + (r + c_off)]);
	    };
}


/******************************************************************
	functions related to active pixels
******************************************************************/

//assign athe function "f" to the value "number"
void gamekit_class::assign_pixelfunction(uint8_t number, uint8_t(*f) (uint8_t, uint8_t, uint32_t))
{
    if ((number <= (gamekit_max_bright + gamekit_pixel_functions_count))
	&& (number > gamekit_max_bright)) {
	pixel_function[number - (gamekit_max_bright + 1)] = f;
    };

}

//this function is assigned to all values at startup, it should never be called
uint8_t dummy_pixel_function(uint8_t row, uint8_t col, uint32_t counter)
{
    return 0;
}

uint8_t pixelfunction16(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 3))
  {
    val++;
    if(val > gamekit_max_bright)
      val = 0;
    old_counter= systemcounter;
  };

  return val;
}

uint8_t pixelfunction17(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 8;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 4))
  {
    val++;
    if(val > gamekit_max_bright)
      val = 0;
    old_counter= systemcounter;
  };

  return val;
}

// pixelfunction for cursor; is dimming up and down
uint8_t pixelfunction18(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;
  static uint8_t up_down = 0;

  if(systemcounter >= (old_counter + 3))
  {
    if (up_down == 0)
    {
      val++;
      if(val == gamekit_max_bright)
        up_down = 1;
    }
    else
    {
      val--;
      if(val == 0)
        up_down = 0;
    };
    old_counter= systemcounter;
  };

  return val;
}

// pixelfunction for cursor; is dimming up and down
uint8_t pixelfunction19(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;
  static uint8_t up_down = 0;

  if(systemcounter >= (old_counter + 5))
  {
    if (up_down == 0)
    {
      val++;
      if(val == gamekit_max_bright)
        up_down = 1;
    }
    else
    {
      val--;
      if(val == 0)
        up_down = 0;
    };
    old_counter= systemcounter;
  };

  return val;
}

// pixelfunction for goal; just switching fast on and off
uint8_t pixelfunction20(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 3))
  {
    if (val == 0)
      val = 15;
    else
      val = 0;

    old_counter= systemcounter;
  };

  return val;
}

uint8_t pixelfunction21(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 30))
  {
    if (val == 0)
      val = 15;
    else
      val = 0;

    old_counter= systemcounter;
  };

  return val;
}

uint8_t pixelfunction22(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 80))
  {
    if (val == 0)
      val = 15;
    else
      val = 0;

    old_counter= systemcounter;
  };

  return val;
}


/******************************************************************
	miscellaneous functions
******************************************************************/

//return the systemcounter
uint32_t gamekit_class::get_systemcounter()
{
    return system_counter;
}

/******************************************************************
	system functions
******************************************************************/


const uint8_t gamma_corr[gamekit_max_bright + 1] = {
    0,				// wird nie benutzt
    180 + (uint8_t) (gamekit_gamma_fact * 7),	// dunkelste
    180 + (uint8_t) (gamekit_gamma_fact * 6),
    180 + (uint8_t) (gamekit_gamma_fact * 5),

    180 + (uint8_t) (gamekit_gamma_fact * 4),
    180 + (uint8_t) (gamekit_gamma_fact * 3),
    180 + (uint8_t) (gamekit_gamma_fact * 2),
    180 + (uint8_t) (gamekit_gamma_fact * 1),

    180,
    180 - (uint8_t) (gamekit_gamma_fact * 1),
    180 - (uint8_t) (gamekit_gamma_fact * 2),
    180 - (uint8_t) (gamekit_gamma_fact * 3),

    180 - (uint8_t) (gamekit_gamma_fact * 4),
    180 - (uint8_t) (gamekit_gamma_fact * 5),
    180 - (uint8_t) (gamekit_gamma_fact * 6),
    180 - (uint8_t) (gamekit_gamma_fact * 7),	//hellste
};


// function to handle time overflow interupt; the most of the hardware related thinks are done here
void gamekit_class::overflow()
{
    static uint8_t tmp_frame[gamekit_col_count][gamekit_row_count];

    static uint8_t state_UP = 0;
    static uint8_t state_DOWN = 0;
    static uint8_t state_LEFT = 0;
    static uint8_t state_RIGHT = 0;
    static uint8_t state_FUNCA = 0;
    static uint8_t state_FUNCB = 0;

    static uint32_t time_UP = 0;
    static uint32_t time_DOWN = 0;
    static uint32_t time_LEFT = 0;
    static uint32_t time_RIGHT = 0;
    static uint32_t time_FUNCA = 0;
    static uint32_t time_FUNCB = 0;

/*
states of buttons:

0: button is not pressed, set buttons_pp to "1" (for the button); if button is pressed set state to 1
1: waiting first (longer) time; if button is released set state to 0, if time is over set state to 2 and set buttons_pp to "1"
2: waiting the standart repeat time; if button is released set state to 0, if time is over set buttons_pp to "1" and wait again

*/

    TCNT2 = gamma_corr[current_bright];	//204

    current_bright--;

    if (current_bright == 0) {

	gamekit_r1_port &= ~_BV(gamekit_r1);
	gamekit_r2_port &= ~_BV(gamekit_r2);
	gamekit_r3_port &= ~_BV(gamekit_r3);
	gamekit_r4_port &= ~_BV(gamekit_r4);
	gamekit_r5_port &= ~_BV(gamekit_r5);
	gamekit_r6_port &= ~_BV(gamekit_r6);
	gamekit_r7_port &= ~_BV(gamekit_r7);

	gamekit_c1_port &= ~_BV(gamekit_c1);
	gamekit_c2_port &= ~_BV(gamekit_c2);
	gamekit_c3_port &= ~_BV(gamekit_c3);
	gamekit_c4_port &= ~_BV(gamekit_c4);
	gamekit_c5_port &= ~_BV(gamekit_c5);

	current_col++;
	current_bright = gamekit_max_bright;

	if (current_col > gamekit_col_count) {

            static uint8_t was_active = 0;
            static uint8_t freq_counter = 0;

            if(gamekit_soundboard_freq_active)
            {
                freq_counter++;
		if(freq_counter == 10)
                {
                    freq_counter = 0;

                    if(was_active)
                        gamekit_soundboard_freq_valid = 1;

                    gamekit_soundboard_freq_tmp = TCNT1; // save timer value
                    TCNT1 = 0;

                    was_active = 1; 
                }
            }
            else
            {
                gamekit_soundboard_freq_valid = 0;
                was_active = 0;
                freq_counter = 0;
            }

	    system_counter++;

	    DDR(gamekit_r2_port) &= ~_BV(gamekit_r2);
	    DDR(gamekit_r3_port) &= ~_BV(gamekit_r3);
	    DDR(gamekit_r4_port) &= ~_BV(gamekit_r4);
	    DDR(gamekit_r5_port) &= ~_BV(gamekit_r5);
	    DDR(gamekit_r6_port) &= ~_BV(gamekit_r6);
	    DDR(gamekit_r7_port) &= ~_BV(gamekit_r7);

	    gamekit_r2_port |= _BV(gamekit_r2);
	    gamekit_r3_port |= _BV(gamekit_r3);
	    gamekit_r4_port |= _BV(gamekit_r4);
	    gamekit_r5_port |= _BV(gamekit_r5);
	    gamekit_r6_port |= _BV(gamekit_r6);
	    gamekit_r7_port |= _BV(gamekit_r7);

	    current_col = 1;

	    for (uint8_t row = 0; row < gamekit_row_count; row++)
		for (uint8_t col = 0; col < gamekit_col_count; col++)
		    if (frame[col][row] <= gamekit_max_bright)
			tmp_frame[col][row] = frame[col][row];
		    else if (frame[col][row] <= (gamekit_max_bright + gamekit_pixel_functions_count))
			tmp_frame[col][row] = (*pixel_function[frame[col][row] - (gamekit_max_bright + 1)]) (col, row, system_counter);
		    else
			tmp_frame[col][row] = 0;

	    static uint32_t next_event_time = 0;

	    if (melody_is_playing == 1) {

		if (next_event_time == 0) {
		    next_event_time = system_counter;
		};

		if ((next_event_time == system_counter)) {
		    switch (pgm_read_word(&melody_ptr[current_melody_event * 2 + 0])) {
		    
		    case (m_1_dot_note):
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_1_dot_duration);
			current_melody_event++;
			};
			break;
			
		    case (m_1_note):
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_1_duration);
			current_melody_event++;
			};
			break;
		    case (m_2_dot_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_2_dot_duration);
			current_melody_event++;
			};
			break;
		    case (m_2_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_2_duration);
			current_melody_event++;
			};
			break;
		    case (m_4_dot_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_4_dot_duration);
			current_melody_event++;
			};
			break;
		    case (m_4_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_4_duration);
			current_melody_event++;
			};
			break;
		    case (m_8_dot_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_8_dot_duration);
			current_melody_event++;
			};
			break;
		    case (m_8_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_8_duration);
			current_melody_event++;
			};
			break;
		    case (m_16_dot_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_16_dot_duration);
			current_melody_event++;
			};
			break;
		    case (m_16_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_16_duration);
			current_melody_event++;
			};
			break;
		    case (m_32_dot_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_32_dot_duration);
			current_melody_event++;
			};
			break;
		    case (m_32_note):
		    
			{
			start_tone( pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) );
			next_event_time = (uint32_t) (system_counter + m_32_duration);
			current_melody_event++;
			};
			break;
		    case (m_rest):
		    
			{
			stop_tone();
			next_event_time = (uint32_t) (system_counter + pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ));
			current_melody_event++;
			};
			break;
		    case (m_set_volume):
		    
			{
			if(pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] ) == LOUD)
			{
			DDR(PORTD) |= _BV(PD4);
			PORTD |= _BV(PD4);
			}
			else
			{
			DDR(PORTD) &= ~_BV(PD4);
			PORTD &= ~_BV(PD4);
			};
			next_event_time = (uint32_t) (system_counter + 1);
			current_melody_event++;
			};
			break;
		    case (m_goto):
		    
			{
			current_melody_event = pgm_read_word( &melody_ptr[current_melody_event * 2 + 1] );
			next_event_time = (uint32_t) (system_counter + 1);
			};
			break;
		    case (m_stop):
		    
			{
			stop_tone();
			melody_is_playing = 0;
			next_event_time = 0;
			current_melody_event = 0;

			TCCR1A = 0;
			TCCR1B = (1 << CS11) | (1 << CS12);
			gamekit_soundboard_freq_active = 1;

			};
			break;

		    };
		}
	    } else {
		next_event_time = 0;
	    };


	    buttons_raw = ((PINB & (gamekit_buttonmaskB)) | ((PINC & (gamekit_buttonmaskC)) << 1));

	    {			// buttons 
		switch (state_UP) {
		case 0:{
			if ((buttons_raw & butt_UP) == 0) {
			    buttons_pp &= ~butt_UP;
			    time_UP = (uint32_t) (system_counter + gamekit_button_first_waittime);
			    state_UP = 1;
			} else
			    buttons_pp |= butt_UP;
		    };
		    break;
		case 1:{
			if ((buttons_raw & butt_UP) == 0) {
			    if (system_counter == time_UP) {
				buttons_pp &= ~butt_UP;
				time_UP = (uint32_t) (system_counter + gamekit_button_waittime);
				state_UP = 2;
			    };
			} else {
			    buttons_pp |= butt_UP;
			    state_UP = 0;
			};
		    };
		    break;
		case 2:{
			if ((buttons_raw & butt_UP) == 0) {
			    if (system_counter == time_UP) {
				buttons_pp &= ~butt_UP;
				time_UP = (uint32_t) (system_counter + gamekit_button_waittime);
			    };
			} else {
			    buttons_pp |= butt_UP;
			    state_UP = 0;
			};
		    };
		    break;

		}

		switch (state_DOWN) {
		case 0:{
			if ((buttons_raw & butt_DOWN) == 0) {
			    buttons_pp &= ~butt_DOWN;
			    time_DOWN = (uint32_t) (system_counter + gamekit_button_first_waittime);
			    state_DOWN = 1;
			} else
			    buttons_pp |= butt_DOWN;
		    };
		    break;
		case 1:{
			if ((buttons_raw & butt_DOWN) == 0) {
			    if (system_counter == time_DOWN) {
				buttons_pp &= ~butt_DOWN;
				time_DOWN = (uint32_t) (system_counter + gamekit_button_waittime);
				state_DOWN = 2;
			    };
			} else {
			    buttons_pp |= butt_DOWN;
			    state_DOWN = 0;
			};
		    };
		    break;
		case 2:{
			if ((buttons_raw & butt_DOWN) == 0) {
			    if (system_counter == time_DOWN) {
				buttons_pp &= ~butt_DOWN;
				time_DOWN = (uint32_t) (system_counter + gamekit_button_waittime);
			    };
			} else {
			    buttons_pp |= butt_DOWN;
			    state_DOWN = 0;
			};
		    };
		    break;

		}

		switch (state_LEFT) {
		case 0:{
			if ((buttons_raw & butt_LEFT) == 0) {
			    buttons_pp &= ~butt_LEFT;
			    time_LEFT = (uint32_t) (system_counter + gamekit_button_first_waittime);
			    state_LEFT = 1;
			} else
			    buttons_pp |= butt_LEFT;
		    };
		    break;
		case 1:{
			if ((buttons_raw & butt_LEFT) == 0) {
			    if (system_counter == time_LEFT) {
				buttons_pp &= ~butt_LEFT;
				time_LEFT = (uint32_t) (system_counter + gamekit_button_waittime);
				state_LEFT = 2;
			    };
			} else {
			    buttons_pp |= butt_LEFT;
			    state_LEFT = 0;
			};
		    };
		    break;
		case 2:{
			if ((buttons_raw & butt_LEFT) == 0) {
			    if (system_counter == time_LEFT) {
				buttons_pp &= ~butt_LEFT;
				time_LEFT = (uint32_t) (system_counter + gamekit_button_waittime);
			    };
			} else {
			    buttons_pp |= butt_LEFT;
			    state_LEFT = 0;
			};
		    };
		    break;

		}

		switch (state_RIGHT) {
		case 0:{
			if ((buttons_raw & butt_RIGHT) == 0) {
			    buttons_pp &= ~butt_RIGHT;
			    time_RIGHT = (uint32_t) (system_counter + gamekit_button_first_waittime);
			    state_RIGHT = 1;
			} else
			    buttons_pp |= butt_RIGHT;
		    };
		    break;
		case 1:{
			if ((buttons_raw & butt_RIGHT) == 0) {
			    if (system_counter == time_RIGHT) {
				buttons_pp &= ~butt_RIGHT;
				time_RIGHT = (uint32_t) (system_counter + gamekit_button_waittime);
				state_RIGHT = 2;
			    };
			} else {
			    buttons_pp |= butt_RIGHT;
			    state_RIGHT = 0;
			};
		    };
		    break;
		case 2:{
			if ((buttons_raw & butt_RIGHT) == 0) {
			    if (system_counter == time_RIGHT) {
				buttons_pp &= ~butt_RIGHT;
				time_RIGHT = (uint32_t) (system_counter + gamekit_button_waittime);
			    };
			} else {
			    buttons_pp |= butt_RIGHT;
			    state_RIGHT = 0;
			};
		    };
		    break;

		}

		switch (state_FUNCA) {
		case 0:{
			if ((buttons_raw & butt_FUNCA) == 0) {
			    buttons_pp &= ~butt_FUNCA;
			    time_FUNCA = (uint32_t) (system_counter + gamekit_button_first_waittime);
			    state_FUNCA = 1;
			} else
			    buttons_pp |= butt_FUNCA;
		    };
		    break;
		case 1:{
			if ((buttons_raw & butt_FUNCA) == 0) {
			    if (system_counter == time_FUNCA) {
				buttons_pp &= ~butt_FUNCA;
				time_FUNCA = (uint32_t) (system_counter + gamekit_button_waittime);
				state_FUNCA = 2;
			    };
			} else {
			    buttons_pp |= butt_FUNCA;
			    state_FUNCA = 0;
			};
		    };
		    break;
		case 2:{
			if ((buttons_raw & butt_FUNCA) == 0) {
			    if (system_counter == time_FUNCA) {
				buttons_pp &= ~butt_FUNCA;
				time_FUNCA = (uint32_t) (system_counter + gamekit_button_waittime);
			    };
			} else {
			    buttons_pp |= butt_FUNCA;
			    state_FUNCA = 0;
			};
		    };
		    break;

		}

		switch (state_FUNCB) {
		case 0:{
			if ((buttons_raw & butt_FUNCB) == 0) {
			    buttons_pp &= ~butt_FUNCB;
			    time_FUNCB = (uint32_t) (system_counter + gamekit_button_first_waittime);
			    state_FUNCB = 1;
			} else
			    buttons_pp |= butt_FUNCB;
		    };
		    break;
		case 1:{
			if ((buttons_raw & butt_FUNCB) == 0) {
			    if (system_counter == time_FUNCB) {
				buttons_pp &= ~butt_FUNCB;
				time_FUNCB = (uint32_t) (system_counter + gamekit_button_waittime);
				state_FUNCB = 2;
			    };
			} else {
			    buttons_pp |= butt_FUNCB;
			    state_FUNCB = 0;
			};
		    };
		    break;
		case 2:{
			if ((buttons_raw & butt_FUNCB) == 0) {
			    if (system_counter == time_FUNCB) {
				buttons_pp &= ~butt_FUNCB;
				time_FUNCB = (uint32_t) (system_counter + gamekit_button_waittime);
			    };
			} else {
			    buttons_pp |= butt_FUNCB;
			    state_FUNCB = 0;
			};
		    };
		    break;
		}
	    }

	    gamekit_r2_port &= ~_BV(gamekit_r2);
	    gamekit_r3_port &= ~_BV(gamekit_r3);
	    gamekit_r4_port &= ~_BV(gamekit_r4);
	    gamekit_r5_port &= ~_BV(gamekit_r5);
	    gamekit_r6_port &= ~_BV(gamekit_r6);
	    gamekit_r7_port &= ~_BV(gamekit_r7);

	    DDR(gamekit_r2_port) |= _BV(gamekit_r2);
	    DDR(gamekit_r3_port) |= _BV(gamekit_r3);
	    DDR(gamekit_r4_port) |= _BV(gamekit_r4);
	    DDR(gamekit_r5_port) |= _BV(gamekit_r5);
	    DDR(gamekit_r6_port) |= _BV(gamekit_r6);
	    DDR(gamekit_r7_port) |= _BV(gamekit_r7);
	}

	switch (current_col) {
	case 1:{
		gamekit_c1_port |= _BV(gamekit_c1);
	    };
	    break;
	case 2:{
		gamekit_c2_port |= _BV(gamekit_c2);
	    };
	    break;
	case 3:{
		gamekit_c3_port |= _BV(gamekit_c3);
	    };
	    break;
	case 4:{
		gamekit_c4_port |= _BV(gamekit_c4);
	    };
	    break;
	case 5:{
		gamekit_c5_port |= _BV(gamekit_c5);
	    };
	    break;
	}

	TCNT2 = gamma_corr[current_bright];
    }

    if (tmp_frame[current_col - 1][0] == current_bright)
	gamekit_r1_port |= _BV(gamekit_r1);
    if (tmp_frame[current_col - 1][1] == current_bright)
	gamekit_r2_port |= _BV(gamekit_r2);
    if (tmp_frame[current_col - 1][2] == current_bright)
	gamekit_r3_port |= _BV(gamekit_r3);
    if (tmp_frame[current_col - 1][3] == current_bright)
	gamekit_r4_port |= _BV(gamekit_r4);
    if (tmp_frame[current_col - 1][4] == current_bright)
	gamekit_r5_port |= _BV(gamekit_r5);
    if (tmp_frame[current_col - 1][5] == current_bright)
	gamekit_r6_port |= _BV(gamekit_r6);
    if (tmp_frame[current_col - 1][6] == current_bright)
	gamekit_r7_port |= _BV(gamekit_r7);


//    PORTD &= ~_BV(PD4);
}

// functions to initiate gamekit hardware and software
void gamekit_class::Begin()
{

    pixel_function[0] = pixelfunction16;
    pixel_function[1] = pixelfunction17;
    pixel_function[2] = pixelfunction18;
    pixel_function[3] = pixelfunction19;
    pixel_function[4] = pixelfunction20;
    pixel_function[5] = pixelfunction21;
    pixel_function[6] = pixelfunction22;


    for (uint8_t i = 7; i < gamekit_pixel_functions_count; i++)
	pixel_function[i] = dummy_pixel_function;

// switch row and column pins to output and set all to LOW  

    DDR(gamekit_r1_port) |= _BV(gamekit_r1);
    DDR(gamekit_r2_port) |= _BV(gamekit_r2);
    DDR(gamekit_r3_port) |= _BV(gamekit_r3);
    DDR(gamekit_r4_port) |= _BV(gamekit_r4);
    DDR(gamekit_r5_port) |= _BV(gamekit_r5);
    DDR(gamekit_r6_port) |= _BV(gamekit_r6);
    DDR(gamekit_r7_port) |= _BV(gamekit_r7);

    DDR(gamekit_c1_port) |= _BV(gamekit_c1);
    DDR(gamekit_c2_port) |= _BV(gamekit_c2);
    DDR(gamekit_c3_port) |= _BV(gamekit_c3);
    DDR(gamekit_c4_port) |= _BV(gamekit_c4);
    DDR(gamekit_c5_port) |= _BV(gamekit_c5);

    gamekit_r1_port &= ~_BV(gamekit_r1);
    gamekit_r2_port &= ~_BV(gamekit_r2);
    gamekit_r3_port &= ~_BV(gamekit_r3);
    gamekit_r4_port &= ~_BV(gamekit_r4);
    gamekit_r5_port &= ~_BV(gamekit_r5);
    gamekit_r6_port &= ~_BV(gamekit_r6);
    gamekit_r7_port &= ~_BV(gamekit_r7);

    gamekit_c1_port &= ~_BV(gamekit_c1);
    gamekit_c2_port &= ~_BV(gamekit_c2);
    gamekit_c3_port &= ~_BV(gamekit_c3);
    gamekit_c4_port &= ~_BV(gamekit_c4);
    gamekit_c5_port &= ~_BV(gamekit_c5);

//  initiate status variables 

    current_col = gamekit_col_count;	// set to last column to ensure that values are copied 
    current_bright = 0;

    gamekit_button_waittime = 20;
    
    gamekit_button_first_waittime = 50;

    buttons_raw=0xFF; //set to 0xFF to force buttons read as unpressed until first real read in ISR
    
    
// timer2 setup: 	interrupt on overflow
// prescaler = 32 

    TCCR2A = 0;
    TCCR2B = 0;
    TCCR2B |= (_BV(CS21) | _BV(CS20));	// prescale = 32

    TIMSK2 |= _BV(TOIE2);

    TCNT2 = gamma_corr[0];

// setup ADC for soundboard

    ADCSRA = (1 << ADEN) | (1 << ADPS1) | (1 <<ADPS2); 				  //ADC einschalten, entspechenden Teiler setzen (128)
    getadc(5); // dummyread zur init.

// setup T1 for soundboard
 
    gamekit_soundboard_freq_active = 1;
    gamekit_soundboard_freq_valid = 0;

    TCCR1A = 0;
    TCCR1B = (1 << CS11) | (1 << CS12);

//  activate interrupts    
    sei();

    DDR(PORTD) &= ~_BV(PD4);	// set "loudnesspin" to input an pullup off
    PORTD &= ~_BV(PD4);


}

// interrupt service function
ISR(TIMER2_OVF_vect)
{
// just call the member function in gamekit class
    gamekit.overflow();
}

