#ifndef gamekit_h
#define gamekit_h

//#include <wiring.h>
#include <avr/interrupt.h>
#include <avr/io.h>

#define gamekit_r1_port PORTC
#define gamekit_r2_port PORTC
#define gamekit_r3_port PORTB
#define gamekit_r4_port PORTB
#define gamekit_r5_port PORTB
#define gamekit_r6_port PORTB
#define gamekit_r7_port PORTB

#define gamekit_r1 PC1
#define gamekit_r2 PC0
#define gamekit_r3 PB5
#define gamekit_r4 PB4
#define gamekit_r5 PB3
#define gamekit_r6 PB2
#define gamekit_r7 PB0

#define gamekit_c1_port PORTD
#define gamekit_c2_port PORTD
#define gamekit_c3_port PORTC
#define gamekit_c4_port PORTC
#define gamekit_c5_port PORTC

#define gamekit_c1 PD2
#define gamekit_c2 PD3
#define gamekit_c3 PC4
#define gamekit_c4 PC3
#define gamekit_c5 PC2

#define DDR(x) (*(&x - 1))

#define gamekit_col_count 5
#define gamekit_row_count 7
#define gamekit_max_bright 15

#define gamekit_gamma_fact 6

#define gamekit_button_FUNCA	_BV(PB2)
#define gamekit_button_FUNCB	_BV(PB0)
#define gamekit_button_UP	_BV(PB3)
#define gamekit_button_DOWN 	_BV(PB5)
#define gamekit_button_RIGHT	_BV(PC0)
#define gamekit_button_LEFT	_BV(PB4)

#define gamekit_buttonmaskB    (gamekit_button_FUNCA | gamekit_button_FUNCB | gamekit_button_UP | gamekit_button_DOWN | gamekit_button_LEFT)
#define gamekit_buttonmaskC gamekit_button_RIGHT

//declaration of masks for button_raw
#define	butt_UP		_BV(3)
#define	butt_DOWN	_BV(5)
#define	butt_LEFT	_BV(4)
#define	butt_RIGHT	_BV(1)
#define	butt_FUNCA	_BV(2)
#define	butt_FUNCB	_BV(0)

#define butt_mask (butt_UP | butt_DOWN | butt_LEFT | butt_RIGHT | butt_FUNCA | butt_FUNCB)

#define gamekit_pixel_functions_count 32

#define SILENT	0
#define LOUD	1

#define	m_1_dot_duration	(uint16_t) (1.5 * m_1_duration)
#define	m_1_duration		177
#define	m_2_dot_duration	(uint16_t) (0.75 * m_1_duration)
#define	m_2_duration		(uint16_t) (0.5 * m_1_duration)
#define	m_4_dot_duration	(uint16_t) (0.375 * m_1_duration)
#define	m_4_duration		(uint16_t) (0.25 * m_1_duration)
#define	m_8_dot_duration	(uint16_t) (0.1875 * m_1_duration)
#define	m_8_duration		(uint16_t) (0.125 * m_1_duration)
#define	m_16_dot_duration	(uint16_t) (0.09375 * m_1_duration)
#define	m_16_duration		(uint16_t) (0.0625 * m_1_duration)
#define	m_32_dot_duration	(uint16_t) (0.046875 * m_1_duration)
#define	m_32_duration		(uint16_t) (0.03125 * m_1_duration)

#define	m_1_dot_note	0
#define	m_1_note	1
#define	m_2_dot_note	2
#define	m_2_note	3
#define	m_4_dot_note	4
#define	m_4_note	5
#define	m_8_dot_note	6
#define	m_8_note	7
#define	m_16_dot_note	8
#define	m_16_note	9
#define	m_32_dot_note	10
#define	m_32_note	11

#define	m_rest		12

#define m_set_volume	13
#define m_goto		14
#define m_stop		15

#define	m_1_dot_rest	(uint16_t) (1.5 * m_1_duration)
#define	m_1_rest	(uint16_t) (1 * m_1_duration)
#define	m_2_dot_rest	(uint16_t) (0.75 * m_1_duration)
#define	m_2_rest	(uint16_t) (0.5 * m_1_duration)
#define	m_4_dot_rest	(uint16_t) (0.375 * m_1_duration)
#define	m_4_rest	(uint16_t) (0.25 * m_1_duration)
#define	m_8_dot_rest	(uint16_t) (0.1875 * m_1_duration)
#define	m_8_rest	(uint16_t) (0.125 * m_1_duration)
#define	m_16_dot_rest	(uint16_t) (0.09375 * m_1_duration)
#define	m_16_rest	(uint16_t) (0.0625 * m_1_duration)
#define	m_32_dot_rest	(uint16_t) (0.046875 m_1_duration)
#define	m_32_rest	(uint16_t) (0.03125 * m_1_duration)

#define	m_C2		65
#define	m_Bt2		m_C2
#define	m_Ct2		69
#define	m_Db2		m_Ct2
#define	m_D2		73
#define	m_Dt2		78
#define	m_Eb2		m_Dt2
#define	m_E2		82
#define	m_Fb2		m_E2
#define	m_F2		87
#define	m_Et2		m_E2
#define	m_Ft2		92
#define	m_Gb2		m_Ft2
#define	m_G2		98
#define	m_Gt2		104
#define	m_Ab2		m_Gt2
#define	m_A2		110
#define	m_At2		117	//46
#define	m_Bb2		m_At2	
#define	m_B2		123
#define	m_Cb2		m_B2

#define	m_C3		131
#define	m_Bt3		m_C3
#define	m_Ct3		139	//49
#define	m_Db3		m_Ct3
#define	m_D3		147
#define	m_Dt3		156
#define	m_Eb3		m_Dt3
#define	m_E3		165
#define	m_Fb3		m_E3
#define	m_F3		175	//53
#define	m_Et3		m_E3
#define	m_Ft3		185
#define	m_Gb3		m_Ft3
#define	m_G3		196
#define	m_Gt3		208
#define	m_Ab3		m_Gt3
#define	m_A3		220	//57
#define	m_At3		233	//58
#define	m_Bb3		m_At3	
#define	m_B3		247     //59
#define	m_Cb3		m_B3

#define	m_C4		262    //60
#define	m_Bt4		m_C4
#define	m_Ct4		277
#define	m_Db4		m_Ct4
#define	m_D4		294
#define	m_Dt4		311
#define	m_Eb4		m_Dt4
#define	m_E4		330
#define	m_Fb4		m_E4
#define	m_F4		349
#define	m_Et4		m_E4
#define	m_Ft4		370
#define	m_Gb4		m_Ft4
#define	m_G4		392
#define	m_Gt4		415
#define	m_Ab4		m_Gt4
#define	m_A4		440
#define	m_At4		466
#define	m_Bb4		m_At4	
#define	m_B4		494    //71
#define	m_Cb4		m_B4

#define	m_C5		523    //72
#define	m_Bt5		m_C5
#define	m_Ct5		554
#define	m_Db5		m_Ct5
#define	m_D5		587
#define	m_Dt5		622
#define	m_Eb5		m_Dt5
#define	m_E5		659
#define	m_Fb5		m_E5
#define	m_F5		698
#define	m_Et5		m_E5
#define	m_Ft5		740
#define	m_Gb5		m_Ft5
#define	m_G5		784
#define	m_Gt5		831
#define	m_Ab5		m_Gt5
#define	m_A5		880
#define	m_At5		932
#define	m_Bb5		m_At5	
#define	m_B5		988
#define	m_Cb5		m_B5   //83

#define	m_C6		1047    //84
#define	m_Bt6		m_C6
#define	m_Ct6		1109
#define	m_Db6		m_Ct6
#define	m_D6		1175
#define	m_Dt6		1245
#define	m_Eb6		m_Dt6
#define	m_E6		1319
#define	m_Fb6		m_E6
#define	m_F6		1397
#define	m_Et6		m_E6
#define	m_Ft6		1480
#define	m_Gb6		m_Ft6
#define	m_G6		1568
#define	m_Gt6		1661
#define	m_Ab6		m_Gt6
#define	m_A6		1760
#define	m_At6		1865
#define	m_Bb6		m_At6	
#define	m_B6		1976    //95
#define	m_Cb6		m_B6

#define	m_C7		2093    //96
#define	m_Bt7		m_C7
#define	m_Ct7		2217
#define	m_Db7		m_Ct7
#define	m_D7		2349
#define	m_Dt7		2489
#define	m_Eb7		m_Dt7
#define	m_E7		2637
#define	m_Fb7		m_E7
#define	m_F7		2794
#define	m_Et7		m_E7
#define	m_Ft7		2960
#define	m_Gb7		m_Ft7
#define	m_G7		3136
#define	m_Gt7		3322
#define	m_Ab7		m_Gt7
#define	m_A7		3520
#define	m_At7		3792
#define	m_Bb7		m_At7	
#define	m_B7		3951    //107
#define	m_Cb7		m_B7

#define	m_C8		4186    //108
#define	m_Bt8		m_C8
#define	m_Ct8		4439
#define	m_Db8		m_Ct8
#define	m_D8		4699
#define	m_Dt8		4978
#define	m_Eb8		m_Dt8
#define	m_E8		5274
#define	m_Fb8		m_E8
#define	m_F8		5588
#define	m_Et8		m_E8
#define	m_Ft8		5920
#define	m_Gb8		m_Ft8
#define	m_G8		6272
#define	m_Gt8		6645
#define	m_Ab8		m_Gt8
#define	m_A8		7040
#define	m_At8		7459
#define	m_Bb8		m_At8	
#define	m_B8		7902    //119
#define	m_Cb8		m_B8


class gamekit_class {
  public:

    uint8_t get_soundboard_volume();

    uint16_t get_soundboard_frequency();

    void overflow();

    void Begin();

    void set_pixel(uint8_t, uint8_t, uint8_t);

    uint8_t get_pixel(uint8_t, uint8_t);

    void load_image(const uint8_t [gamekit_col_count][gamekit_row_count]);
    
    void load_map(uint8_t *, uint8_t, uint8_t, uint8_t, uint8_t);
    
    void assign_pixelfunction(uint8_t, uint8_t(*)(uint8_t, uint8_t, uint32_t));

    uint32_t get_systemcounter();

    uint8_t get_buttons();

    uint8_t button_pressed(uint8_t);

    void wait_button_pressed(uint8_t);

    void wait_button_released(uint8_t);

    void set_button_timing(uint8_t, uint8_t);
    
    void play_tone(uint16_t, uint16_t, uint8_t);

    void play_melody(uint16_t *);
    
    uint16_t get_current_melody_event();
    
    void set_current_melody_event(uint16_t);

    uint16_t freq2ocr(uint16_t);

    void start_tone(uint16_t freq);

    void stop_tone();
    
  private:

    volatile uint8_t gamekit_soundboard_freq_active;

    volatile uint8_t gamekit_soundboard_freq_valid;

    volatile uint16_t gamekit_soundboard_freq_tmp;

    uint16_t getadc(uint8_t channel);

    volatile uint8_t frame[gamekit_col_count][gamekit_row_count];

    volatile uint8_t buttons_raw;

    volatile uint8_t buttons_pp;

    volatile uint8_t current_col;

    volatile uint8_t current_bright;

    uint8_t(*pixel_function[gamekit_pixel_functions_count]) (uint8_t, uint8_t, uint32_t);

    volatile uint32_t system_counter;
    
    volatile uint8_t melody_is_playing;

    volatile uint16_t *melody_ptr;
    
    volatile uint16_t current_melody_event;
    
    volatile uint8_t gamekit_button_waittime;
    
    volatile uint8_t gamekit_button_first_waittime;

};



extern gamekit_class gamekit;

#endif
