#include <gamekit_2_1.h>
#include <avr/pgmspace.h>

/*
to load a map it has to be "drawn" at first
 
 The declaration begins always with:
 
 uint8_t name[rows][columns] PROGMEM = {
 
 where "name" is the name of the map,
 rows is the number of rows
 columns is the number of columns
 
 then a list of brightness values (or values assigned to pixelfunctions) follows.
 It is smart, but not necessary, to align the rows for better readability.
 
 */

uint8_t matrix[64][7] PROGMEM = {
  0 ,6 ,0 ,0 ,12,9 ,0 ,
  0 ,9 ,0 ,0 ,15,12,0 ,
  0 ,12,3 ,0 ,0 ,15,0 ,
  0 ,15,6 ,0 ,0 ,0 ,0 ,
  3 ,0 ,9 ,0 ,3 ,0 ,0 ,
  6 ,0 ,12,0 ,6 ,0 ,3 ,
  9 ,0 ,15,0 ,9 ,0 ,6 ,
  12,0 ,0 ,3 ,12,0 ,9 ,
  15,0 ,0 ,6 ,15,0 ,12,
  0 ,0 ,0 ,9 ,0 ,0 ,15,
  0 ,3 ,0 ,12,0 ,0 ,0 ,
  3 ,6 ,0 ,15,0 ,3 ,0 ,
  6 ,9 ,0 ,0 ,0 ,6 ,0 ,
  9 ,12,3 ,0 ,0 ,9 ,0 ,
  12,15,6 ,0 ,0 ,12,0 ,
  15,0 ,9 ,0 ,0 ,15,0 ,
  0 ,0 ,12,0 ,0 ,0 ,0 ,
  0 ,0 ,15,0 ,0 ,0 ,3 ,
  0 ,0 ,0 ,3 ,0 ,0 ,6 ,
  0 ,0 ,0 ,6 ,0 ,0 ,9 ,
  3 ,0 ,0 ,9 ,0 ,3 ,12,
  6 ,0 ,0 ,12,0 ,6 ,15,
  9 ,0 ,0 ,15,3 ,9 ,0 ,
  12,0 ,3 ,0 ,6 ,12,0 ,
  15,0 ,6 ,0 ,9 ,15,3 ,
  0 ,0 ,9 ,0 ,12,0 ,6 ,
  3 ,0 ,12,3 ,15,0 ,9 ,
  6 ,0 ,15,6 ,0 ,0 ,12,
  9 ,0 ,0 ,9 ,0 ,0 ,15,
  12,0 ,0 ,12,0 ,7,0 ,
  15,3 ,0 ,15,0 ,6,0 ,
  0 ,6 ,0 ,0 ,3 ,9 ,0 ,
  0 ,9 ,0 ,0 ,6 ,12,0 ,
  0 ,12,0 ,0 ,9 ,15,0 ,
  0 ,15,0 ,0 ,12,0 ,0 ,
  0 ,0 ,0 ,0 ,15,0 ,0 ,
  0 ,0 ,3 ,0 ,0 ,0 ,0 ,
  0 ,0 ,6 ,0 ,0 ,3 ,0 ,
  0 ,0 ,9 ,0 ,0 ,6 ,0 ,
  0 ,0 ,12,0 ,3 ,9 ,0 ,
  0 ,0 ,15,3 ,6 ,12,0 ,
  3 ,0 ,0 ,6 ,9 ,15,0 ,
  6 ,0 ,0 ,9 ,12,0 ,0 ,
  9 ,0 ,0 ,12,15,0 ,0 ,
  12,0 ,0 ,15,0 ,0 ,3 ,
  15,0 ,3 ,0 ,0 ,0 ,6 ,
  0 ,0 ,6 ,0 ,0 ,0 ,9 ,
  0 ,0 ,9 ,0 ,0 ,0 ,12,
  0 ,0 ,12,3 ,0 ,0 ,15,
  0 ,3 ,15,6 ,0 ,3 ,0 ,
  0 ,6 ,0 ,9 ,0 ,6 ,0 ,
  0 ,9 ,0 ,12,0 ,9 ,0 ,
  0 ,12,0 ,15,0 ,12,0 ,
  3 ,15,0 ,0 ,0 ,15,0 ,
  6 ,0 ,3 ,0 ,0 ,0 ,0 ,
  9 ,0 ,6 ,0 ,0 ,0 ,0 ,
  12,0 ,9 ,0 ,3 ,0 ,0 ,
  15,0 ,12,0 ,6 ,3 ,0 ,
  0 ,3 ,15,0 ,9 ,6 ,0 ,
  0 ,6 ,0 ,0 ,12,9 ,0 ,
  0 ,9 ,0 ,0 ,15,12,0 ,
  0 ,12,3 ,0 ,0 ,15,0 ,
  0 ,15,6 ,0 ,0 ,0 ,0 ,
  3 ,0 ,9 ,0 ,3 ,0 ,0
};

void setup(){
  gamekit.Begin();

  Serial.begin(9600);
  Serial.println("begin");
}


/*
a cutout of the map (wich has to be larger than the display) is loaded with the gamekit.load_map funtion and the following arguments:
 
 gamekit.load_map( (uint_t *) name, rows, columns, row_offset, column_offset)
 
 name: this is the name of the image, (uint_t *) has to be be added always
 rows: the number rows of the whole map 
 columns: the number columns of the whole map 
 these three can be found in the declaration of the map
 
 row_offset
 column_offset
 are the distances from the upper and left borders of the map to the region to be loaded.
 
 example:
 
 this is a map with 20 columns and 12 rows and a row offset of 4 and a column offset of 8
 
 O O O O O O O O O O O O /\  O O O O O O
 O O O O O O O O O O O O || row    O O O
 O O O O O O O O O O O O || offset O O O
 O O O O O O O O O O O O \/  O O O O O O
 O O O O O O O | --------------| O O O O 
 O O O O O O O | O O O O O O O | O O O O
 <===========> | O O O O O O O | O O O O
 O column  O O | O O O O O O O | O O O O
 O offset  O O | O O O O O O O | O O O O
 O O O O O O O | O O O O O O O | O O O O
 O O O O O O O |---------------| O O O O
 O O O O O O O O O O O O O O O O O O O O


in the following code the map "matrix" is shifted over the display.
Note that the first (offset = 0) and the last (offset = 59) five lines are the same.
This is the trick of the uninteruppted "flow" of the map over the display.
*/


uint8_t i = 59;

void loop(){

  delay(200);

  i--;

  if( i == 0 )
    i = 59;

  gamekit.load_map( (uint8_t *) matrix, 64, 7, i, 0);

  Serial.println(i , DEC);

}
