#include <gamekit_2_1.h>
#include <avr/pgmspace.h>

/* at first the melody is defined

the declaration always begins with:

uint16_t name[][2] PROGMEM =
{

where "name" is the name of the melody.
Then a list of events an corresponding arguments follows.

examples for events:

play a eighth note:
m_8_note

play a eighth dottednote:
m_8_dot_note

rest some time:
m_rest

examples for arguments corresponding to play note events:

C in 5rd oktave:
m_C5

-------------------I----------
                   I
-------------------I----------
                  OI
------------------------------

------------------------------

------------------------------



C in 5rd oktave with a #:
m_Ct5  (the # symbol can't be used, therfore it is replaced by a t)

-------------------I----------
                   I
-------------------I----------
                 #OI
------------------------------

------------------------------

------------------------------

C in 5rd oktave with a b:
m_Cb5

-------------------I----------
                   I
-------------------I----------
                 bOI
------------------------------

------------------------------

------------------------------

see documentation for further informations*/

uint16_t melody[][2] PROGMEM =
{
  //------------ 1
  m_set_volume, LOUD,
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  //---------- 2
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  m_4_dot_note, m_D4,
  m_rest      , m_8_rest,
  //---------- 3
  m_8_dot_note, m_A4,
  m_rest      , m_16_rest,
  m_8_dot_note, m_A4,
  m_rest      , m_16_rest,
  m_8_dot_note, m_A4,
  m_rest      , m_16_rest,
  m_8_dot_note, m_Bb4,      //this is the 23th event
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  //----------- 4
  m_8_dot_note, m_Ct4,
  m_rest      , m_16_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  m_4_dot_note, m_D4,
  m_rest      , m_8_rest,
  //----------- 5
  m_8_dot_note, m_D5,
  m_rest      , m_16_rest,
  m_8_note    , m_D4,
  m_rest      , m_16_rest,
  m_16_note   , m_D4,
  m_8_note    , m_D5,
  m_rest      , m_8_rest,
  m_8_note    , m_Ct5,
  m_rest      , m_16_rest,
  m_16_note    , m_C5,
  //------------ 6
  m_16_note   , m_Bb4,
  m_16_note   , m_At4,
  m_8_note    , m_Bb4,
  m_rest      , m_8_rest,
  m_16_note   , m_Dt4,
  m_rest      , m_16_rest,
  m_8_note    , m_Gt4,
  m_rest      , m_8_rest,
  m_8_note    , m_G4,
  m_rest      , m_16_rest,
  m_16_note   , m_Ft4,
  //------------ 7
  m_16_note   , m_F4,
  m_16_note   , m_E4,
  m_8_note    , m_F4,
  m_rest      , m_8_rest,
  m_16_note   , m_Bb3,
  m_rest      , m_16_rest,
  m_8_note    , m_Ct4,
  m_rest      , m_8_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_C4,
  //------------ 8  
  m_8_dot_note, m_F4,
  m_rest      , m_16_rest,
  m_8_note    , m_D4,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  m_4_dot_note, m_A4,
  m_rest      , m_8_rest,
  //------------ 9
  m_8_dot_note, m_D5,
  m_rest      , m_16_rest,
  m_8_note    , m_D4,
  m_rest      , m_16_rest,
  m_16_note   , m_D4,
  m_8_note    , m_D5,
  m_rest      , m_8_rest,
  m_8_note    , m_Ct5,
  m_rest      , m_16_rest,
  m_16_note   , m_C5,
  //------------ 10
  m_16_note   , m_Bb4,
  m_16_note   , m_At4,
  m_8_note    , m_Bb4,
  m_rest      , m_8_rest,
  m_16_note   , m_Dt4,
  m_rest      , m_16_rest,
  m_8_note    , m_Gt4,
  m_rest      , m_8_rest,
  m_8_note    , m_G4,
  m_rest      , m_16_rest,
  m_16_note   , m_Ft4,
  //------------ 11
  m_16_note   , m_F4,
  m_16_note   , m_E4,
  m_8_note    , m_F4,
  m_rest      , m_8_rest,
  m_16_note   , m_Bb3,
  m_rest      , m_16_rest,
  m_8_note    , m_Ct4,
  m_rest      , m_8_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  //------------ 12
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  m_4_dot_note, m_D4,
  m_stop      , 0
};


void setup(){
  gamekit.Begin();

  Serial.begin(9600);
  Serial.println("begin");

  // give the address of "melody" to the play_melody function to start playback
  gamekit.play_melody((uint16_t *) melody);
}

uint16_t old_event = 0;

void loop(){

//print current event number to the serial interface (just for debuging)
  if( gamekit.get_current_melody_event() != old_event)  
  {
    Serial.println(gamekit.get_current_melody_event(), DEC);
    old_event = gamekit.get_current_melody_event();
  };
 
//if "Funktion A" button is pressed jump to 23th event 
  if(gamekit.button_pressed(butt_FUNCA))
    gamekit.set_current_melody_event(23);

//if "Funktion A" button is pressed play "melody" again
  if(gamekit.button_pressed(butt_FUNCB))
    gamekit.play_melody((uint16_t *) melody);

}
